public class MergeSorter {

  /**
   * Sorts an array of items using merge-sort.
   */
  public static void sort(int[] items) {
    mergesortHelper(items, 0, items.length);
  }

  /**
   * Runs merge-sort on a portion of the items, starting
   * at the "left" item, up to (but not including) the "right" item.
   * 
   * @param items - entire array to be sorted.
   * @param left  - left index of the subarray to be sorted.
   * @param right - right index of the subarray to be sorted.
   */
  private static void mergesortHelper(int[] items, int left, int right) {
    int n = right - left;
    if (n <= 1) {
      return;
    }

    // recursively call merge sort on left and right subarrays
    int mid = left + n / 2;
    mergesortHelper(items, left, mid);
    mergesortHelper(items, mid, right);

    // merge the two subarrays
    merge(items, left, mid, right);
  }

  /**
   * Merges the sorted subarrays items[left:mid], items[mid:right] into
   * items[left:right] so that items[left:right] is sorted.
   * 
   * @param items - the entire array being sorted.
   * @param left  - index of the 1st element in the left subarray.
   * @param mid   - index of the 1st element in the right subarray.
   * @param right - 1 + the index of the last element in the right subarray.
   */
  private static void merge(int[] items, int left, int mid, int right) {
    // retrieve the minimum of (items[iL], items[iR]) until one subarray is empty
    int iL = left;
    int iR = mid;
    int[] merged = new int[right - left];
    int j = 0;
    while (iL < mid && iR < right) {
      if (items[iL] < items[iR]) {
        merged[j] = items[iL];
        iL++;
      } else {
        merged[j] = items[iR];
        iR++;
      }
      j++;
    }

    // TODO 1: retrieve any remaining items from the left subarray

    // TODO 2: retrieve any remaining items from the right subarray

    // TODO 3: place the sorted items (in merged) back in the array (in items)

  }

  /**
   * Prints the subarray of items from left -> right,
   * excluding the item at right.
   * 
   * @param items - complete array of items to print.
   * @param left  - index of the first item to print.
   * @param right - index of the last item to print + 1.
   */
  static void printItems(int[] items, int left, int right) {
    for (int i = left; i < right; i++) {
      System.out.print(items[i]);
      if (i + 1 < right) {
        System.out.print(", ");
      }
    }
    System.out.println();
  }

  /**
   * Tests the merge-sort implementation.
   * 
   * @param args - unused.
   */
  public static void main(String[] args) {

    int[] items = { 6, 5, 3, 1, 8, 7, 2, 4 };
    sort(items);
    for (int x : items) {
      System.out.println(x);
    }
  }
}